#ifndef INC_GTLIST_H
#define INC_GTLIST_H

#include "gtwidget.h"
#include "gtcblist.h"
#include "gtdlink.h"

/************************************************************************\
*
* GtList class header.
*
* $Ident: This file is part of the 'Gt-library' 1.0.0 $
* $Copyright: (C) 1997 by Gran Gustafsson $
*
\************************************************************************/

/* Type of list item */
typedef struct GtListItem
{
	GtDLink lnk;
	char *str;
	int len;
	ulong color;
	sbool sel;
} GtListItem;

/* Type of callback data for select */
typedef struct GtListSelectCD
{
	GtListItem *item;
	word multi;
	sbool sel;
} GtListSelectCD;

/* Type of callback data for change */
typedef struct GtListChangeCD
{
	ulong top;
	ulong visible;
	ulong total;
} GtListChangeCD;

/* List modes */
#define GTLM_READ				0
#define GTLM_SINGLE			1
#define GTLM_MULTI			2
#define GTLM_ALWAYSONE	3
#define GTLM_LAST_NR		3


class GtList : public GtWidget
{
public:
		/* Standard */
	GtList(void);
	void Destroy(void);
	sbool Init(char *n,GtManager *p,GtTag *tgs);
	sbool Set(GtTag *tgs);
	void Get(GtTag *tgs);
	char *GetClass(void) { return("GtList"); }
	sbool IsSubClassOf(char *cn);
		/* Values */
	void SetEMask(ulong em);
		/* Callbacks */
	void AddCB(ulong tag,GtCallB cb,GtUDPtr ud);
	void RemCB(ulong tag,GtCallB cb);
		/* Item management */
	void ClrListItems(void);
	void AddItems(GtDLink *lhd,ulong pos);
	sbool AddItem(GtListItem *l,ulong pos);
	sbool AddItem(char *str,ulong pos);
	sbool AddItem(char *str,ulong pos,ulong col);
	void RemItem(GtListItem *l);
	void RemItem(ulong pos);
	void RemItem(char *str);
	void SelectItem(GtListItem *l);
	void SelectItem(ulong pos);
	void SelectItem(char *str);
	GtListItem *GetSelected(GtListItem *l);
	GtListItem *GetNextPrevItem(GtListItem *l,sbool next);
	void ShowItem(GtListItem *l);
	ulong FindItemPos(GtListItem *l);
		/* Internal method */
	void SetPosSize(word xp,word yp,word xs,word ys);
protected:
		/* Internal methods */
	sbool Draw(XEvent *xe);
	sbool Mouse(XEvent *xe);
protected:
		/* Instance variables */
	GtCBList sel,chg;
	XRectangle rect;
	GtDLink ihd;
	GtListItem *topi,*seli,*curri;
	ulong topnr,visnr,totnr;
	int cy;
	Time last;
	word space,foff,fh,mclk;
	ubyte mode,lflags,ftype,ibw,ibdrw,bdrw;
};

extern GtList *GtMkList(char *n,GtManager *p,GtTag *tgs);
extern GtList *GtMkList(char *n,GtManager *p,ulong tag,...);

#endif /* INC_GTLIST_H */
